<?php
/**
* 2007-2022 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2022 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if(!defined('_PS_VERSION_')) {
    exit;
}

use Google\Client;
use Google\Service\Sheets\Sheet;
use Google\Service\Sheets\ValueRange;
use Google\Auth\Subscriber\ScopedAccessTokenSubscriber;

require_once(_PS_MODULE_DIR_).'addifygooglesheetsconnector/classes/addifyGoogleSheetsConnectorClass.php';

class Addifygooglesheetsconnector extends Module
{
    public function __construct()
    {
        $this->name ='addifygooglesheetsconnector';
        $this->tab = 'front_office_features';
        $this->version = "1.0.0";
        $this->author = 'Addify';

        $this->bootstrap = true;
 
        $this->need_instance = 0;
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);

        parent::__construct();

        $this->displayName = $this->l('Addify Google Sheets Connector');
        $this->description = $this->l("This module sends all the orders data according to the fields of Google Sheets.");

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        if (!Configuration::get('addifygooglesheetsconnector')) {
            $this->warning = $this->l('No name provided');
        }
    }
    // ----------------------------------------------------------------------------------------------------------------------------
    public function install()
    {
        Configuration::updateValue('AGSC_LIVE_MODE', false);
        Configuration::updateValue('AGSC_API_TOKEN', false);

        include(dirname(__FILE__).'/sql/install.php'); 
        return parent::install() && $this->installTab() && $this->registerHook('displayOrderConfirmation') && $this->registerHook('actionGetAdminOrderButtons') && $this->registerHook('actionValidateOrder') && $this->registerHook('actionOrderStatusUpdate');
    }

    public function uninstall()
    {
        Configuration::deleteByName('AGSC_LIVE_MODE');
        Configuration::deleteByName('AGSC_API_TOKEN');

        include(dirname(__FILE__).'/sql/uninstall.php');
        return parent::uninstall() && $this->uninstallTab() && $this->unregisterHook('displayOrderConfirmation') && $this->unregisterHook('actionGetAdminOrderButtons') && $this->unregisterHook('actionValidateOrder') && $this->unregisterHook('actionOrderStatusUpdate');
    }
    // -----------------------------------------------------------------------
    public function getContent()
    {
        if ( (bool)Tools::isSubmit('statusaddifygooglesheetsconnectortb')) 
        {
            $agscStatusUpdate = AddifyGoogleSheetsConnectorClass::agscStatusUpdate('User_active_mode', (int)Tools::getValue('sheets_id'));
            
            if (!$agscStatusUpdate) {
                $this->context->controller->errors[] = $this->l('Status update failed.');
            } 
            return $this->rendorForm(); 
        }

        if (((bool)Tools::isSubmit('addifyGoogleSheetsConnectorBtn'.$this->name)) == true) {
            $this->addifyUpdateGoogleConnectorGeneralFormValues();   
        }


        if (((bool)Tools::isSubmit('addaddifygooglesheetsconnector')) == true) {
            return $this->rendorForm();
        }


        if (((bool)Tools::isSubmit('addifyGoogleSheetsConnectorUserFormBtn'.$this->name)) == true) {
            $this->addifyUpdateGoogleSheetsConnectorUserFormValues();
        }


        if(Tools::isSubmit('updateaddifygooglesheetsconnectortb')) {
            return $this->rendorForm();
        }

        if (Tools::isSubmit('deleteaddifygooglesheetsconnectortb')) {
            if (!Validate::isLoadedObject($myid = new AddifyGoogleSheetsConnectorClass((int)Tools::getValue('sheets_id')))) {
                
            } else {

                if (!$myid->delete()) {
                    $this->context->controller->errors[] = $this->l('Rule cannot be deleted.');
                } 
                else 
                {
                    $this->context->controller->confirmations[] = $this->l('Rule deleted successfully.');
                }
            }
        }

        if (Tools::isSubmit('submitBulkdeleteaddifygooglesheetsconnectortb')) {
             
            $bulkact = Tools::getValue('addifygooglesheetsconnectortbBox');
            if (isset($bulkact) && $bulkact) {
               
                $deleted = 0;
                foreach ($bulkact as $bulkchange) {
                    if (!Validate::isLoadedObject($hello = new AddifyGoogleSheetsConnectorClass((int)$bulkchange))) {
                        
                        break;
                    } 
                    else {

                        if ($hello->delete()) {
                            $deleted++;
                        }
                    }
                }
                if (count($this->context->controller->errors)) {
                    $this->context->controller->errors;
                } else {
                    $this->context->controller->confirmations[] = sprintf($this->l('Rules deleted successfully.'), $deleted);
                }
            }
        }
       

        return $this->addifyGoogleSheetsConnectorDisplayGeneralForm().$this->helperListing();
    }
    // ---------------------------------------------------------------------------------------------------------//
    public function addifyGoogleSheetsConnectorDisplayGeneralForm()
    {
        $addfiyGoogleSheetsConnectorDisplayFormVar = array(
                 
            'form' => array(
                    'legend' => array(
                    'title' => $this->l('General Settings'),
                    'icon' => 'icon-cogs'
                     ),

                'input' => array(

                array( 
                        'type' => 'switch',
                        'label' => $this->l('Active Mode'),
                        'name' => 'AGSC_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Use this module in live mode'),

                        'values' => array(

                                array(
                                    'id' => 'active_on',
                                    'value' => 1,
                                    'label' => $this->l('Enabled')
                                ),

                                 array(
                                    'id' => 'active_off',
                                    'value' => 0,
                                    'label' => $this->l('Disabled')
                                )
                            ),

                ),

                    array(
                    'type' => 'personal_api_token',
                    'label' => $this->l('Personal API token'),
                    'name'=> 'AGSC_API_TOKEN',
                    'required' => true
                    ),

                ),
 
                'submit' => array(
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right'
                )
            ),
                
        );
    // ----------------------------------------------------------------------------------------------------------//
    $helper = new HelperForm();

    $helper->show_toolbar = false;
    $helper->table = $this->table;
    $helper->module = $this;
    $helper->default_form_language = $this->context->language->id;
    $helper->identifier = $this->identifier;
    $helper->submit_action = 'addifyGoogleSheetsConnectorBtn'.$this->name;

    $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;

    $helper->token = Tools::getAdminTokenLite('AdminModules');
        
    require_once (_PS_MODULE_DIR_).'addifygooglesheetsconnector/google-api-php-client/vendor/autoload.php';

    $client = new Google_Client();
    $client->setApplicationName('SpreadsheetConnector');
    $client->setScopes([\Google_Service_Sheets::SPREADSHEETS]);
    $client->setAuthConfig( __DIR__ . '/credentials.json' );
    $client->setAccessType('offline');
    $client->setPrompt('select_account consent');
   
    $helper->tpl_vars = array(

    'fields_value' => $this->addifyGetGoogleSheetsConnectorGeneralFormValues(),
    'AGSC_authUrl_value' => $client->createAuthUrl(), 

    'languages' => $this->context->controller->getLanguages(),
    'id_language' => $this->context->language->id,
    );

    return $helper->generateForm(array($addfiyGoogleSheetsConnectorDisplayFormVar));

    }
    // -------------------------------------------------------------------------------------------------------//
    public function addifyGetGoogleSheetsConnectorGeneralFormValues()
    {
       if(Tools::isSubmit('addifyGoogleSheetsConnectorBtn'.$this->name) == false) {
            return array(
            'AGSC_LIVE_MODE' => Configuration::get('AGSC_LIVE_MODE'),
            'AGSC_API_TOKEN' => Configuration::get('AGSC_API_TOKEN'),
            );
        }
        else {
            return array(
            'AGSC_LIVE_MODE'=> Tools::getValue('AGSC_LIVE_MODE'),
            'AGSC_API_TOKEN' => Tools::getValue('AGSC_API_TOKEN'),
            );
        }
    }
    // ---------------------------------------------------------------------------------------------------------//
    public function addifyUpdateGoogleConnectorGeneralFormValues()
    {
        $addifyUpdateGoogleSheetsConnector = $this->addifyGetGoogleSheetsConnectorGeneralFormValues();
            
        configuration::updateValue('AGSC_LIVE_MODE',$addifyUpdateGoogleSheetsConnector['AGSC_LIVE_MODE']);
        configuration::updateValue('AGSC_API_TOKEN',$addifyUpdateGoogleSheetsConnector['AGSC_API_TOKEN']);

        Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);   
    }
    //-----------------------------------------------------------------------------------------------------------//
    protected function helperListing()
    {
        $addifyFields_list = array(
         'sheets_id' => array(
            'title' => $this->l('ID'),
            'type' => 'text',
            'search' => false,
         ),

                    'agsc_title' => array(
                        'title' => $this->l('Title'),
                        'type' => 'text',
                         'search' => false,
                    ),

                    'agsc_priority' => array(
                        'title' => $this->l('Priority'),
                        'type' => 'text',
                         'search' => false,
                    ),

                     'User_active_mode' => array(
                        'title' => $this->l('Status'),
                        'type' => 'bool',
                        'active'=>'status',
                        'search' => false,
                     ),

                     'agsc_add_date' => array(
                        'title' => $this->l('Add Date'),
                        'type' => 'date',
                         'search' => false,
                     ),

                     'agsc_update_date' => array(
                        'title' => $this->l('Update Date'),
                        'type' => 'date',
                         'search' => false,
                     ),
        );
    //----------------------------------------------------------------------------------------------------------//
    $helper = new HelperList();
    $helper->shopLinkType = '';
    $helper->simple_header = false;
    $helper->show_toolbar = true;
    $helper->identifier = 'sheets_id';
    $helper->actions = array('edit', 'delete');
    $helper->bulk_actions = array(
                                  'delete' => array(
                                  'text' => $this->l('Delete selected'),
                                  'icon' => 'icon-trash',
                                   'confirm' => $this->l('Delete selected items?')
                                    )
                                 );

    $helper->listTotal = count(AddifyGoogleSheetsConnectorClass::getListContent());
    $helper->toolbar_btn['new'] = array('href' => AdminController::$currentIndex.'&configure='.$this->name.'&add'.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules'),
        'desc' => $this->l('Add new'));
    $helper->title = 'Addify Google Sheets Connector Rules';
    $helper->table = "addifygooglesheetsconnectortb";
    $helper->token = Tools::getAdminTokenLite('AdminModules');
    $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
    return $helper->generateList(AddifyGoogleSheetsConnectorClass::getListContent(),$addifyFields_list);
    }
    //----------------------------------------------------------------------------------------------------------//
    public function rendorForm()
    {
        $agsc_options = array(
                array(
                    'option' => 'id_order', 
                    'name' => 'ID_Order' 
                    ),   
                  array(
                    'option' => 'reference',
                    'name' => 'Reference'
                  ),
                  array(
                    'option' => 'id_shop_group',     
                    'name' => 'ID_Shop_Group'    
                  ),
                  array(
                    'option' => 'id_shop',
                    'name' => 'ID_Shop'
                  ),
                  array(
                    'option' => 'id_carrier',
                    'name' => 'ID_Carrier'
                  ),
                  array(
                    'option' => 'id_lang',
                    'name' => 'ID_Lang'
                  ),
                  array(
                    'option' => 'id_customer',
                    'name' => 'ID_Customer'
                  ),
                  array(
                    'option' => 'id_cart',
                    'name' => 'ID_Cart'
                  ),
                  array(
                    'option' => 'id_currency',
                    'name' => 'ID_Currency'
                  ),
                  array(
                    'option' => 'id_address_delivery',
                    'name' => 'ID_Address_Delivery'
                  ),
                  array(
                    'option' => 'id_address_invoice',
                    'name' => 'ID_Address_Invoice'
                  ),
                  array(
                    'option' => 'current_state',
                    'name' => 'Current_State'
                  ),
                  array(
                    'option' => 'secure_key',
                    'name' => 'Secure_Key'
                  ),

                   array(
                    'option' => 'payment',
                    'name' => 'Payment'
                  ),
                    array(
                    'option' => 'conversion_rate',
                    'name' => 'Conversion_Rate'
                  ),
                     array(
                    'option' => 'module',
                    'name' => 'Module'
                  ),
                      array(
                    'option' =>  'recyclable',
                    'name' => 'Recyclable'
                  ),
                       array(
                    'option' => 'gift',
                    'name' => 'Gift'
                  ),
                        array(
                    'option' => 'gift_message',
                    'name' => 'Gift_Message'
                  ),
                         array(
                    'option' => 'mobile_theme',
                    'name' => 'Mobile_Theme'
                  ),
                          array(
                    'option' => 'shipping_number',
                    'name' => 'shipping_number'
                  ),
                           array(
                    'option' => 'total_discounts',
                    'name' => 'Total_Discounts'
                  ),
                            array(
                    'option' => 'total_discounts_tax_incl',
                    'name' => 'Total_Discounts_Tax_Incl'
                  ),
                             array(
                    'option' => 'total_discounts_tax_excl',
                    'name' => 'Total_Discounts_Tax_Excl'
                  ),
                     array(
                    'option' => 'total_paid',
                    'name' => 'Total_Paid'
                  ),
                    array(
                    'option' => 'total_paid_tax_incl',
                    'name' => 'Total_Paid_Tax_Incl'
                  ),
                     array(
                    'option' => 'total_paid_tax_excl',
                    'name' => 'Total_Paid_Tax_Excl'
                  ),
                      array(
                    'option' => 'total_paid_real',
                    'name' => 'Total_Paid_Real'
                  ),
                    array(
                    'option' => 'total_products',
                    'name' => 'Total_Products'
                  ),
                   array(
                    'option' => 'total_products_wt',
                    'name' => 'Total_Products_Wt'
                  ),
                   array(
                    'option' => 'total_shipping',
                    'name' => 'Total_Shipping'
                  ),
                    array(
                    'option' => 'total_shipping_tax_incl',
                    'name' => 'Total_Shipping_Tax_Incl'
                  ),
                 array(
                    'option' => 'total_shipping_tax_excl',
                    'name' => 'Total_Shipping_Tax_Excl'
                  ),
                  array(
                    'option' => 'carrier_tax_rate',
                    'name' => 'Carrier_Tax_Rate'
                  ),
                 array(
                    'option' => 'total_wrapping',
                    'name' => 'Total_Wrapping'
                  ),
                  array(
                    'option' => 'total_wrapping_tax_incl',
                    'name' => 'Total_Wrapping_Tax_Incl'
                  ),
                 array(
                    'option' => 'total_wrapping_tax_excl',
                    'name' => 'Total_Wrapping_Tax_Excl'
                  ),
                  array(
                    'option' => 'round_mode',
                    'name' => 'Round_Mode'
                  ),
                   array(
                    'option' => 'round_type',
                    'name' => 'Round_Type'
                  ),
                 array(
                    'option' => 'invoice_number',
                    'name' => 'Invoice_Number'
                  ),
                  array(
                    'option' => 'delivery_number',
                    'name' => 'Delivery_Number'
                  ),
                   array(
                    'option' => 'invoice_date',
                    'name' => 'Invoice_Date'
                  ),
                 array(
                    'option' => 'delivery_date',
                    'name' => 'Delivery_Date'
                  ),
                  array(
                    'option' => 'valid',
                    'name' => 'Valid'
                  ),
                   array(
                    'option' => 'date_add',
                    'name' => 'Date_Add'
                  ),
                 array(
                    'option' => 'date_upd',
                    'name' => 'Date_Upd'
                  ),
                 array(
                    'option' => 'note',
                    'name' => 'Note'
                  )
            );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = 'addifygooglesheetsconnectortb';
        $helper->show_cancel_button = false;
        $helper->module = $this;
        $helper->identifier = 'sheets_id';
        
        $helper->submit_action = 'addifyGoogleSheetsConnectorUserFormBtn'.$this->name;
        $helper->back_url = $helper->currentIndex.'&token='.$helper->token;

        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        $helper->tpl_vars = array(
            'fields_value' => $this->addifyGetGoogleSheetsConnectorUserFormValues(),
            'fields_options' => $agsc_options,
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'context_link' => Context::getContext()->link,
        );

        return $helper->generateForm(array($this->addifyGoogleSheetsConnectorDisplayUserForm())); 
    }

    //----------------------------------------------------------------------------------------------------------//
    public function addifyGoogleSheetsConnectorDisplayUserForm()
    {

    return  array(
           'form' => array(
                
                'legend' => array(
                'title' => $this->l('Rule Settings'),
                'icon' => 'icon-cogs'
            ),

                'input'=> array(

                    array(
                        'type' => 'hidden',
                        'name'=>'sheets_id',

                    ),

                    array( 
                        'type' => 'switch',
                        'label' => $this->l('Active'),
                        'name' => 'User_active_mode',
                        'is_bool' => true,
                        'desc' => $this->l('Enable/Disable this rule'),

                        'values' => array(

                                array(
                                    'id' => 'active_on',
                                    'value' => 1,
                                    'label' => $this->l('Enabled')
                                ),

                                 array(
                                    'id' => 'active_off',
                                    'value' => 0,
                                    'label' => $this->l('Disabled')
                                )
                            ),

                    ),

                        array(
                            'type' => 'text',
                            'label' => $this->l('Title'),
                            'name' => 'agsc_title',
                         ),

                        array(
                            'type' => 'text',
                            'label' => $this->l('Priority'),
                            'name' => 'agsc_priority',
                            'desc' => $this->l('Value must be numeric.'),
                            'required' => true  
                         ),

                    array(
                    'type' => 'text',
                    'label' => $this->l('Link Google Sheets'),
                    'name'=> 'agsc_link',
                    'desc' => $this->l('Before google sheet link fill, spreadsheet general access link must be accessable and set to the editor.'),
                    'required' => true
                    ),

                    array(
                    'type' => 'text',
                    'label' => $this->l('Sheet Name'),
                    'name'=> 'agsc_name',
                    'required' => true
                    ),

                array(
                        'type' => 'type_map_form_fields',
                        'name' => 'agsc_select_fields[]',
                        'label' => $this->l('Map Form Fields'),
                        'required' => true
                ),

            ),

                'submit' => array(
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-success pull-right'
                ),

                'buttons' => array(
                'cancel' => array(
                'title' => $this->l('Cancel'),
                'class' => 'btn btn-default',
                'icon' => 'process-icon-cancel',
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules'),
                    ),
                )
 
            ),

        );
    }
    //----------------------------------------------------------------------------------------------------------//
    public function addifyGetGoogleSheetsConnectorUserFormValues()
    {
        $addifyGetUserFormValues = array();

        if(Tools::getValue('sheets_id') && Tools::isSubmit('addifyGoogleSheetsConnectorUserFormBtn'.$this->name) === false) {

            $addifyGetUserForm =  new AddifyGoogleSheetsConnectorClass((int)Tools::getValue('sheets_id'));
  
            $addifyGetUserFormValues['sheets_id'] = $addifyGetUserForm->sheets_id;
            $addifyGetUserFormValues['User_active_mode'] = $addifyGetUserForm->User_active_mode;
            $addifyGetUserFormValues['agsc_title'] = $addifyGetUserForm->agsc_title;
            $addifyGetUserFormValues['agsc_priority'] = $addifyGetUserForm->agsc_priority;
            $addifyGetUserFormValues['agsc_link'] = $addifyGetUserForm->agsc_link;
            $addifyGetUserFormValues['agsc_name'] = $addifyGetUserForm->agsc_name;
            $addifyGetUserFormValues['agsc_select_fields'] = explode(',',$addifyGetUserForm->addifyGoogleSheetsConnector_select_field);
        }

        else {
            
            $addifyGetUserFormValues['sheets_id'] = Tools::getValue('sheets_id');
            $addifyGetUserFormValues['User_active_mode'] = Tools::getValue('User_active_mode');
            $addifyGetUserFormValues['agsc_title'] = Tools::getValue('agsc_title');
 
            $addifyGetUserFormValues['agsc_priority'] = Tools::getValue('agsc_priority');
            $addifyGetUserFormValues['agsc_link'] = Tools::getValue('agsc_link');
            $addifyGetUserFormValues['agsc_name'] = Tools::getValue('agsc_name');
              
            if (!Tools::getValue('agsc_select_fields')) {
            $addifyGetUserFormValues['agsc_select_fields'] = '';
            } else {
            $addifyGetUserFormValues['agsc_select_fields'] = implode(',',Tools::getValue('agsc_select_fields'));
            }

        }

        return $addifyGetUserFormValues;
    }
    //----------------------------------------------------------------------------------------------------------//
    public function addifyUpdateGoogleSheetsConnectorUserFormValues()
    {
        $addifyUpadateUserFormValues = $this->addifyGetGoogleSheetsConnectorUserFormValues();

        if (!is_numeric($addifyUpadateUserFormValues['agsc_priority']) || !$addifyUpadateUserFormValues['agsc_priority']) {
            $this->context->controller->errors[] = $this->l('Priority value must be numeric');
        }

        if (empty($addifyUpadateUserFormValues['agsc_link']) || !strpos($addifyUpadateUserFormValues['agsc_link'], 'edit#gid')){
            $this->context->controller->errors[] = $this->l('Google Sheet link must be Correct and not be Empty');
        }

        if (empty($addifyUpadateUserFormValues['agsc_name'])) {
            $this->context->controller->errors[] = $this->l('Empty Sheet Name');
        }

        if (empty($addifyUpadateUserFormValues['agsc_select_fields'])) {
            $this->context->controller->errors[] = $this->l('Empty Map Form Field');
        }

        if (count($this->context->controller->errors)) {
           return $this->context->controller->errors;
        }


        if (($addifyUpadatevar = Tools::getValue('sheets_id'))) {
            $call_back = 'update';
            $addifyGoogleSheetsData = new AddifyGoogleSheetsConnectorClass((int)$addifyUpadatevar);
            $addifyGoogleSheetsData->agsc_update_date = date('Y-m-d');

            if (Configuration::get('AGSC_LIVE_MODE') == 1 )
            {
                if($addifyUpadateUserFormValues['User_active_mode'] == 1)
                { // update bold heading
                $this->agscDeleteSheetsValues($addifyUpadateUserFormValues['agsc_link'],$addifyUpadateUserFormValues['agsc_name']);
                $this->agscBatchUpdateSheetsValues($addifyUpadateUserFormValues['agsc_link'],$addifyUpadateUserFormValues['agsc_name'],$addifyUpadateUserFormValues['agsc_select_fields']);
                }
            }
        } 
        else 
        {
            $call_back = 'add';
            $addifyGoogleSheetsData = new AddifyGoogleSheetsConnectorClass();
            $addifyGoogleSheetsData->agsc_add_date = date('Y-m-d');

            if (Configuration::get('AGSC_LIVE_MODE') == 1 )
            {
                if($addifyUpadateUserFormValues['User_active_mode'] == 1)
                { // add bold heading
                $this->agscDeleteSheetsValues($addifyUpadateUserFormValues['agsc_link'],$addifyUpadateUserFormValues['agsc_name']);
                $this->agscAppendSheetsValues($addifyUpadateUserFormValues['agsc_link'],$addifyUpadateUserFormValues['agsc_name'],$addifyUpadateUserFormValues['agsc_select_fields']);
                }
            }
        }

        $addifyGoogleSheetsData->sheets_id = $addifyUpadateUserFormValues['sheets_id'];
        $addifyGoogleSheetsData->User_active_mode = $addifyUpadateUserFormValues['User_active_mode'];
        $addifyGoogleSheetsData->agsc_title = $addifyUpadateUserFormValues['agsc_title'];
        $addifyGoogleSheetsData->agsc_priority = $addifyUpadateUserFormValues['agsc_priority'];
        $addifyGoogleSheetsData->agsc_link = $addifyUpadateUserFormValues['agsc_link'];
        $addifyGoogleSheetsData->agsc_name = $addifyUpadateUserFormValues['agsc_name']; 
        $addifyGoogleSheetsData->addifyGoogleSheetsConnector_select_field  = $addifyUpadateUserFormValues['agsc_select_fields'];

        if (count($this->context->controller->errors)) {
            return $this->context->controller->errors;
        }

        if (!call_user_func(array($addifyGoogleSheetsData, $call_back))) {
            $this->context->controller->errors = sprintf($this->l('Something went wrong while performing operation %s'), $call_back);
        } 

        else {

            if($call_back=='add') {

                if (Configuration::get('AGSC_LIVE_MODE') == 1 ){
                    if($addifyUpadateUserFormValues['User_active_mode'] == 1){
                      $this->agscSpecificAppendSheetsAddValues(); // add orders data on sheet
                    }
                }
                
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }

            else {

                if (Configuration::get('AGSC_LIVE_MODE') == 1 ){
                    if($addifyUpadateUserFormValues['User_active_mode'] == 1){
                        $this->agscSpecificAppendSheetsUpdateValues(Tools::getValue('sheets_id')); // update orders data on sheet
                    }
                }

            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }

          
        }   
    }
    //---------------------------------------------------------------------------------------//
    public function installTab()
    {
        $tab = new Tab();
        $tab->active = 1;
        $tab->id_parent = 0;
        $tab->class_name = 'AdminParentsaddifygooglesheetsconnector';
        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $this->l('Addify Google Sheets Connector');
        }
        $tab->add();
        $tab = new Tab();
        $tab->active = 1;
        $tab->id_parent = (int)Tab::getIdFromClassName('AdminParentsaddifygooglesheetsconnector');
        $tab->class_name = 'addifyGoogleSheetsConnectorAdminCont';
        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $this->l('Manage Google Sheets Connector');
        }
        $tab->module = $this->name;
        return $tab->add();
    }

    public function uninstallTab()
    {
        // Delete parent
        $tabIdP = (int) Tab::getIdFromClassName('AdminParentsaddifygooglesheetsconnector');
        if (!$tabIdP) {
            return true;
        }
        $tabP = new Tab($tabIdP);
        $tabP->delete();

        // delete child
        $tabId = (int) Tab::getIdFromClassName('addifyGoogleSheetsConnectorAdminCont');
        if (!$tabId) {
            return true;
        }

        $tab = new Tab($tabId);
        $tab->delete();

        return true;
    }
    //---------------------------------------------------------------------------------------//
    public function getAuthenticationKey()
    {
        require_once (_PS_MODULE_DIR_).'addifygooglesheetsconnector/google-api-php-client/vendor/autoload.php';
        $client = new Google_Client();
        $client->setApplicationName('SpreadsheetConnector');
        $client->setScopes([\Google_Service_Sheets::SPREADSHEETS]);
        $client->setAuthConfig( __DIR__ . '/credentials.json' );
        $client->setAccessType('offline');
        $client->setPrompt('select_account consent');

        // Load previously authorized token from a file, if it exists.
        // The file token.json stores the user's access and refresh tokens, and is
        // created automatically when the authorization flow completes for the first time.

        $tokenPath = __DIR__ . '/token.json';
        
        if (file_exists($tokenPath)) {
            $authUrl = $client->createAuthUrl();
        
            $accessToken = Tools::file_get_contents($tokenPath);
            $client->setAccessToken($accessToken);
        }
        // If there is no previous token or it's expired.
        if ($client->isAccessTokenExpired()) {
            // Refresh the token if possible, else fetch a new one.
            if ($client->getRefreshToken()) {
                $client->fetchAccessTokenWithRefreshToken($client->getRefreshToken());
            } else {
            //Request authorization from the user.
                $authUrl = $client->createAuthUrl();
                
                // here we get token from configuration table 
                $AGSC_API_TOKEN = Configuration::get('AGSC_API_TOKEN');

                // dynamic token save in this var
                $authCode = $AGSC_API_TOKEN;
                
                // Exchange authorization code for an access token.
                $accessToken = $client->fetchAccessTokenWithAuthCode($authCode);
                
                $client->setAccessToken($accessToken);

                // Check to see if there was an error.
                if (array_key_exists('error', $accessToken)) {
                    throw new Exception(join(', ', $accessToken));
                }
            }
            // Save the token to a file.
            if (!file_exists(dirname($tokenPath))) {
                mkdir(dirname($tokenPath), 0700, true);
            }
            file_put_contents($tokenPath, json_encode($client->getAccessToken()));
        }

        return $client;
    }

    //--------------------------------------------------------------------------------------------------------------//
    public  function agscAppendSheetsValues($agsc_link,$agsc_sheet_name,$agsc_select_fields) {

        $client = $this->getAuthenticationKey();

        $sheets = new Google_Service_Sheets($client);
            
        //----------------------------------------//  
        try {

        $agsc_select_fields = explode(",",$agsc_select_fields);
        $agsc_all_values = [$agsc_select_fields];

        $spid = explode('/', $agsc_link);
        $spreadsheetId = $spid[5];

        $range = $agsc_sheet_name;

        $spreadsheetsRangeID = explode('/', $agsc_link);
        $spreadsheetRangeID = $spreadsheetsRangeID[6]; 

        $spreadsheetsRange = explode('=', $spreadsheetRangeID);
        $ssRangeID = $spreadsheetsRange[1]; 

        //---------------------------------------//
        $myRange = [
            'sheetId' => $ssRangeID, 
            'startRowIndex' => 0,
            'endRowIndex' => 1,
        ];

        // define the formatting, change background color and bold text
        $format = [
            'textFormat' => [
              'bold' => true,
            ]
        ];

        // build request
        $requests = [
            new Google_Service_Sheets_Request([
                'repeatCell' => [
                    'fields' => 'userEnteredFormat.backgroundColor, userEnteredFormat.textFormat.bold',
                    'range' => $myRange,
                    'cell' => [
                        'userEnteredFormat' => $format,
                    ],
                ],
            ])
        ];

        //--------------------------------------//
        $updateBody = new Google_Service_Sheets_ValueRange([
            'values' =>  $agsc_all_values,
        ]);

        $params = ['valueInputOption' => 'Raw'];
        //---------------------------------------//
        $batchUpdateCellFormatRequest = new Google_Service_Sheets_BatchUpdateSpreadsheetRequest([
            'requests' => $requests,
        ]);

          $sheets->spreadsheets->batchUpdate($spreadsheetId, $batchUpdateCellFormatRequest);
          $sheets->spreadsheets_values->append($spreadsheetId, $range, $updateBody, $params);
        } 
        catch(Exception $e) {
        $this->context->controller->errors[] = $this->l('Please verify that your sheet link and sheet name is valid then save the rule.');
        }  
    }
    //--------------------------------------------------------------------------------------------------------------//
    public function agscBatchUpdateSheetsValues($agsc_link,$agsc_sheet_name,$agsc_select_field)
    {
        $client = $this->getAuthenticationKey();
        $sheets = new Google_Service_Sheets($client);
        try {
            $spid = explode('/', $agsc_link);
            $spreadsheetId = $spid[5];
            $range = $agsc_sheet_name;
            $spreadsheetsRangeID = explode('/', $agsc_link);
            $spreadsheetRangeID = $spreadsheetsRangeID[6]; 
            $spreadsheetsRange = explode('=', $spreadsheetRangeID);
            $ssRangeID = $spreadsheetsRange[1]; 

            //-----------------------------------------------------//      
            $myRange = [
                'sheetId' => $ssRangeID, 
                'startRowIndex' => 0,
                'endRowIndex' => 1,
            ];

            $format = [
                'textFormat' => [
                  'bold' => true,
                ]
            ];

            $requests = [
                new Google_Service_Sheets_Request([
                    'repeatCell' => [
                        'fields' => 'userEnteredFormat.backgroundColor, userEnteredFormat.textFormat.bold',
                        'range' => $myRange,
                        'cell' => [
                            'userEnteredFormat' => $format,
                        ],
                    ],
                ])
            ];
            //-----------------------------------------------------//      
            $agsc_select_fields = explode(",", $agsc_select_field);
            $values = [$agsc_select_fields];
            $data[] = new Google_Service_Sheets_ValueRange([
                'range' =>  $range,
                'values' =>  $values,
            ]);
            //-----------------------------------------------------//      
            $batchUpdateRequest = new Google_Service_Sheets_BatchUpdateValuesRequest([
               'valueInputOption' => 'Raw',
               'data' => $data,
            ]);
            //-----------------------------------------------------//
            $batchUpdateCellFormatRequest = new Google_Service_Sheets_BatchUpdateSpreadsheetRequest([
                'requests' => $requests,
            ]);

            //-----------------------------------------------------//
            $sheets->spreadsheets->batchUpdate($spreadsheetId, $batchUpdateCellFormatRequest);
            $sheets->spreadsheets_values->batchUpdate($spreadsheetId, $batchUpdateRequest);
        } 

        catch(Exception $sheet) {
        $this->context->controller->errors[] = $this->l('Please verify that your sheet link and sheet name is valid then edit the rule.');
        }
    }
    //--------------------------------------------------------------------------------------------------------------//
    public function agscDeleteSheetsValues($agsc_link,$agsc_sheet_name)
    {
        $client = $this->getAuthenticationKey();
        $service = new Google_Service_Sheets($client);

        try {
            $spid = explode('/', $agsc_link);
            $spreadsheetId = $spid[5];
            $range = $agsc_sheet_name;
            $requestBody = new Google_Service_Sheets_ClearValuesRequest();
            $result = $service->spreadsheets_values->clear($spreadsheetId, $range, $requestBody);
        } catch (Exception $e) {
            $this->context->controller->errors[] = $this->l('Please verify that your sheet link and sheet name is valid.');
        }     
    } 
    //--------------------------------------------------------------------------------------------------------------//
    public function agscSpecificAppendSheetsAddValues()
    {
        $client = $this->getAuthenticationKey();
        $sheets = new Google_Service_Sheets($client);

        try {

            $agscGetOrdersDatas = AddifyGoogleSheetsConnectorClass::agscGetOrdersTbDataForAppendSpecificValues();

            $agscGetRuleData = AddifyGoogleSheetsConnectorClass::agscGetRuleAddData();
            
                foreach ($agscGetRuleData as $agscGetRuleDataValueId) {

                    $agsc_link  = $agscGetRuleDataValueId['agsc_link'];

                    $agsc_sheet_name  = $agscGetRuleDataValueId['agsc_name'];

                    $agsc_select_fields = explode(",",$agscGetRuleDataValueId['addifyGoogleSheetsConnector_select_field']);    

                    $fill_keys = [];
                    foreach ($agsc_select_fields as $key => $agsc_select_field) {
                        foreach ($agscGetOrdersDatas as $key => $agscGetOrdersData) {

                                $valueNew = " ";
                                array_push($agscGetOrdersData,$valueNew);

                                if (empty($agscGetOrdersData['gift_message'])) {
                                    $agscGetOrdersData['gift_message'] = $valueNew;
                                }
                                if (empty($agscGetOrdersData['shipping_number'])) {
                                    $agscGetOrdersData['shipping_number'] = $valueNew;
                                }

                                if (empty($agscGetOrdersData['note'])) {
                                    $agscGetOrdersData['note']  = $valueNew;
                                }
                                
                            $fill_keys[$key][$agsc_select_field] = $agscGetOrdersData[$agsc_select_field];

                        }
                    }

                   
                    $spid = explode('/', $agsc_link);

                    $spreadsheetId = $spid[5];

                    $range = $agsc_sheet_name; 

                    foreach ($fill_keys as $key => $fill_key) {

                        $values = [array_values($fill_key)];
                
                        $updateBody = new Google_Service_Sheets_ValueRange([
                            'values' =>  $values,
                        ]);

                        $params = ['valueInputOption' => 'Raw'];

                        $sheets->spreadsheets_values->append($spreadsheetId, $range, $updateBody, $params);
                    }
                }
            } 
            catch (Exception $e) {
                
            }        
    }
    //--------------------------------------------------------------------------------------------------------------//
    public function agscSpecificAppendSheetsUpdateValues($sheets_id)
    {
        $client = $this->getAuthenticationKey();

        $sheets = new Google_Service_Sheets($client);

        try {
            $agscGetRuleData = AddifyGoogleSheetsConnectorClass::agscGetRuleUpdateData($sheets_id);

                $agscGetOrdersDatas = AddifyGoogleSheetsConnectorClass::agscGetOrdersTbDataForAppendSpecificValues();

                foreach ($agscGetRuleData as $agscGetRuleDataValue) 
                {
                    $agsc_link  = $agscGetRuleDataValue['agsc_link'];
                    $agsc_sheet_name  = $agscGetRuleDataValue['agsc_name'];
                    $agsc_select_fields = explode(",",$agscGetRuleDataValue['addifyGoogleSheetsConnector_select_field']);
               

                    $fill_keys = [];
                    foreach ($agsc_select_fields as $key => $agsc_select_field) 
                    {
                        foreach ($agscGetOrdersDatas as $key => $agscGetOrdersData) 
                        {
                                $valueNew = " ";
                                array_push($agscGetOrdersData,$valueNew);

                                if (empty($agscGetOrdersData['gift_message'])) {
                                    $agscGetOrdersData['gift_message'] = $valueNew;
                                }
                                if (empty($agscGetOrdersData['shipping_number'])) {
                                    $agscGetOrdersData['shipping_number'] = $valueNew;
                                }

                                if (empty($agscGetOrdersData['note'])) {
                                    $agscGetOrdersData['note']  = $valueNew;
                                }

                            $fill_keys[$key][$agsc_select_field] = $agscGetOrdersData[$agsc_select_field];
                        }
                    }
               
                        $spid = explode('/', $agsc_link);
                        $spreadsheetId = $spid[5];
                        $range = $agsc_sheet_name; 

                    foreach ($fill_keys as $key => $fill_key) 
                    {

                        $values = [array_values($fill_key)];

                        $updateBody = new Google_Service_Sheets_ValueRange([
                            'values' =>  $values,
                        ]);

                        $params = ['valueInputOption' => 'Raw'];

                        $sheets->spreadsheets_values->append($spreadsheetId, $range, $updateBody, $params);
                    }  
                }   
        } 
        catch (Exception $e) {
            
        }        
    }
    //----------   front order  ---------------------------------------------//
    public function agscFrontOrdersCompletionAppendSheetsValues()
    {
        $client = $this->getAuthenticationKey();

        $sheets = new Google_Service_Sheets($client);

        try {
           $agscGetRule = AddifyGoogleSheetsConnectorClass::agscGetRule(); 

           $agscGetOrdersDatas = AddifyGoogleSheetsConnectorClass::agscGetOrdersDataFront();

           foreach ($agscGetRule as $agscGetRuleDataValue) {

            $agsc_link  = $agscGetRuleDataValue['agsc_link'];
            $agsc_sheet_name  = $agscGetRuleDataValue['agsc_name'];

            $agsc_select_fields = explode(",",$agscGetRuleDataValue['addifyGoogleSheetsConnector_select_field']);

            $fill_keys = [];
            foreach ($agsc_select_fields as $key => $agsc_select_field) {

                foreach ($agscGetOrdersDatas as $key => $agscGetOrdersData) {

                     $valueNew = " ";
                                array_push($agscGetOrdersData,$valueNew);

                                if (empty($agscGetOrdersData['gift_message'])) {
                                    $agscGetOrdersData['gift_message'] = $valueNew;
                                }
                                if (empty($agscGetOrdersData['shipping_number'])) {
                                    $agscGetOrdersData['shipping_number'] = $valueNew;
                                }

                                if (empty($agscGetOrdersData['note'])) {
                                    $agscGetOrdersData['note']  = $valueNew;
                                }
                                
                    $fill_keys[$key][$agsc_select_field] = $agscGetOrdersData[$agsc_select_field];
                }
            }

          $spid = explode('/', $agsc_link);

          $spreadsheetId = $spid[5];

          $range = $agsc_sheet_name; 

            $values = [array_values($fill_keys[0])];

            $updateBody = new Google_Service_Sheets_ValueRange([
                'values' =>  $values,
            ]);

            $params = ['valueInputOption' => 'Raw'];

            $response = $sheets->spreadsheets_values->append($spreadsheetId, $range, $updateBody, $params);
          }
        } 
        catch (Exception $e) 
        {
            
        }  
    }

    //----------  front order called ----------------------------//
    public function hookDisplayOrderConfirmation()
    {
        if (Configuration::get('AGSC_LIVE_MODE') == 0) {
            return;
        }

        $this->agscFrontOrdersCompletionAppendSheetsValues();
    }

    //-------------  back order  -------------------//
    public function agscBackOrdersCompletionAppendSheetsValues()
    {
        $client = $this->getAuthenticationKey();
        $sheets = new Google_Service_Sheets($client);

        try {
           $agscGetRule = AddifyGoogleSheetsConnectorClass::agscGetRule(); 
           $agscGetOrdersDatas = AddifyGoogleSheetsConnectorClass::agscGetOrdersDataBack();

           foreach ($agscGetRule as $agscGetRuleDataValue) {
            $agsc_link  = $agscGetRuleDataValue['agsc_link'];
            $agsc_sheet_name  = $agscGetRuleDataValue['agsc_name'];
            $agsc_select_fields = explode(",",$agscGetRuleDataValue['addifyGoogleSheetsConnector_select_field']);

            $fill_keys = [];
            foreach ($agsc_select_fields as $key => $agsc_select_field) {

                foreach ($agscGetOrdersDatas as $key => $agscGetOrdersData) {

                    $valueNew = " ";
                            array_push($agscGetOrdersData,$valueNew);

                            if (empty($agscGetOrdersData['gift_message'])) {
                                $agscGetOrdersData['gift_message'] = $valueNew;
                            }
                            if (empty($agscGetOrdersData['shipping_number'])) {
                                $agscGetOrdersData['shipping_number'] = $valueNew;
                            }

                            if (empty($agscGetOrdersData['note'])) {
                                $agscGetOrdersData['note']  = $valueNew;
                            }
                                
                    $fill_keys[$key][$agsc_select_field] = $agscGetOrdersData[$agsc_select_field];
                }
            }

          $spid = explode('/', $agsc_link);

          $spreadsheetId = $spid[5];

          $range = $agsc_sheet_name; 

            $values = [array_values($fill_keys[0])];

            $updateBody = new Google_Service_Sheets_ValueRange([
                'values' =>  $values,
            ]);

            $params = ['valueInputOption' => 'Raw'];

            $response = $sheets->spreadsheets_values->append($spreadsheetId, $range, $updateBody, $params);
          }
        } 
        catch (Exception $e) 
        {
            
        }  
    }

    //------------- back order update status -------------------//
    public function agscBackOrdersUpdateStatusCompletionAppendSheetsValues($orderId, $current_order_refrence_number)
    {
        $client = $this->getAuthenticationKey();
        $sheets = new Google_Service_Sheets($client);

        try {
           $agscGetRule = AddifyGoogleSheetsConnectorClass::agscGetRule(); 
           $agscGetOrdersDatas = AddifyGoogleSheetsConnectorClass::agscGetOrdersDataBackOrderStatusUpdate($orderId, $current_order_refrence_number);

           foreach ($agscGetRule as $agscGetRuleDataValue) {
            $agsc_link  = $agscGetRuleDataValue['agsc_link'];
            $agsc_sheet_name  = $agscGetRuleDataValue['agsc_name'];
            $agsc_select_fields = explode(",",$agscGetRuleDataValue['addifyGoogleSheetsConnector_select_field']);

                $fill_keys = [];
                foreach ($agsc_select_fields as $key => $agsc_select_field) {
                    foreach ($agscGetOrdersDatas as $key => $agscGetOrdersData) {

                        $valueNew = " ";
                        array_push($agscGetOrdersData,$valueNew);
                        if (empty($agscGetOrdersData['gift_message'])) {
                            $agscGetOrdersData['gift_message'] = $valueNew;
                        }
                        if (empty($agscGetOrdersData['shipping_number'])) {
                            $agscGetOrdersData['shipping_number'] = $valueNew;
                        }
                        if (empty($agscGetOrdersData['note'])) {
                            $agscGetOrdersData['note']  = $valueNew;
                        }   
                        $fill_keys[$key][$agsc_select_field] = $agscGetOrdersData[$agsc_select_field]; 
                    }
                }

                $spid = explode('/', $agsc_link);
                $spreadsheetId = $spid[5];
                $range = $agsc_sheet_name; 
                $values = [array_values($fill_keys[0])];
                $updateBody = new Google_Service_Sheets_ValueRange([
                    'values' =>  $values,
                ]);
                $params = ['valueInputOption' => 'Raw'];
                $response = $sheets->spreadsheets_values->append($spreadsheetId, $range, $updateBody, $params);
            }
        } 
        catch (Exception $e) 
        {
            
        }  
    }

    //-------------  back order & back order update status called  ----------------//
    public function hookActionOrderStatusUpdate($params) 
    {
        if (Configuration::get('AGSC_LIVE_MODE') == 0) {
            return;
        }
        
        // Check if a session is active
        if (session_status() == PHP_SESSION_NONE) {
            session_start();
        }
        $agsc_currentUrl = $_SERVER['REQUEST_URI'];
        $_SESSION['myVariable'] = $agsc_currentUrl;  // Store the variable in a session

        if (strpos($agsc_currentUrl, "admin") !== false) {

            $agscCurentUrl = explode('/', $agsc_currentUrl);
            if ($agscCurentUrl) {
                $current_order_id = $agscCurentUrl[6]; // update order id

                $orderId = $params['id_order']; 
                $order = new Order($orderId);

                $current_order_refrence_number = $order->reference; // id refrence 
                $current_order_status = $order->current_state; // id current state

                if ($current_order_status) {

                    $agsc_targetString_order_status = 'status?_token';
                    if (strpos($agsc_currentUrl, $agsc_targetString_order_status) !== false) { // when we update order status
                        $this->agscBackOrdersUpdateStatusCompletionAppendSheetsValues($orderId, $current_order_refrence_number);
                    }
                } 
            }
        }    
    }
    //----------------------  called back order  -----------------------------------//
    public function hookActionGetAdminOrderButtons($params) 
    {
        if (Configuration::get('AGSC_LIVE_MODE') == 0) {
            return;
        }
        
        if (session_status() == PHP_SESSION_NONE) {  // Check if a session is active
            session_start();
        }
        if (isset($_SESSION['myVariable'])) {
            $myVariable_new_order_URL = $_SESSION['myVariable'];   // Now you can use $myVariable in this hook
            $agsc_targetString_order_status = 'place?_token';
            if (strpos($myVariable_new_order_URL, $agsc_targetString_order_status) !== false) {
                $this->agscBackOrdersCompletionAppendSheetsValues();
                session_destroy();
            }
        }
    }
    // --------------------------------------------------------------------------------//
}